#ifndef __BS_ARRAY_H__
#define __BS_ARRAY_H__

#include "bs_types.h"
#include "bs_assert.h"

typedef struct bs_array_st 
{
  bs_uint32_t size, capacity; 
  void **pArray; 
} bs_array_t; 

bs_array_t* 
bs_array_create(bs_uint32_t capacity); 

void 
bs_array_destroy(bs_array_t *pArray); 

void 
bs_array_ensure_capacity(bs_array_t *pArray); 

void 
bs_array_trim(bs_array_t *pArray); 

#define BS_ARRAY_CREATE(CAPACITY) \
  bs_array_create(CAPACITY)

#define BS_ARRAY_DESTROY(ARRAY) \
  bs_array_destroy(ARRAY)

#define BS_ARRAY_GET(ARRAY, INDEX) \
  (BS_ASSERT((ARRAY) != NULL), \
   BS_ASSERT((INDEX) < (ARRAY)->size), \
   BS_ASSERT((INDEX) >= 0), \
   (ARRAY)[INDEX]) 

#define BS_ARRAY_SET(ARRAY, INDEX, VALUE) \
  BS_ASSERT((ARRAY) != NULL); \
  BS_ASSERT((INDEX) < (ARRAY)->size); \
  BS_ASSERT((INDEX) >= 0); \
  (ARRAY)[INDEX] = (VALUE) 

#define BS_ARRAY_ADD(ARRAY, VALUE) \
  BS_ASSERT((ARRAY) != NULL); \
  bs_array_ensure_capacity(ARRAY); \
  (ARRAY)[(ARRAY)->size] = (VALUE); \
  (ARRAY)->size++ 

#define BS_ARRAY_GET_SIZE(ARRAY) \
  (ARRAY)->size;

#define BS_ARRAY_TRIM(ARRAY) \
  bs_array_trim(ARRAY)

#endif
