#include <math.h>

#include "bs_blob.h"
#include "bs_mem.h"
#include "bs_assert.h"
#include "bs_util.h"
#include "bs_vector.h"

static bs_int32_t 
bs_blob_get_num_pointmasses(bs_float_t radius)
{  
  return 40;
}

static bs_int32_t 
bs_blob_clamp_index(bs_int32_t index, 
                    bs_int32_t highBound)
{
  while(index < 0)
  {
    index = highBound + index; 
  }
  while(index >= highBound)
  {
    index = index - highBound; 
  }
  
  return index; 
}

bs_blob_t* 
bs_blob_create(bs_float_t xPos, 
               bs_float_t yPos, 
               bs_float_t radius)
{
  bs_blob_t *pBlob; 
  bs_int32_t i, p; 
  
  BS_MEM_ALLOC_TYPE(pBlob, bs_blob_t, 1);   

  pBlob->pMiddle = bs_pointmass_create(xPos, yPos, 1.0); 
  pBlob->pComputedMiddle = bs_vector_create(xPos, yPos); 
  pBlob->radius = radius; 
  pBlob->faceStyle = BS_BLOB_FACE_STYLE_MOUTH_CLOSED; 
  pBlob->eyeStyle = BS_BLOB_EYE_STYLE_OPEN; 

  pBlob->pMiddle = bs_pointmass_create(xPos, yPos, 1.0f); 
  pBlob->spA = bs_vector_create(0.1f, 0.1f); 
  pBlob->spB = bs_vector_create(0.2f, 0.2f); 

  pBlob->numPointmasses = bs_blob_get_num_pointmasses(radius); 
  pBlob->numSticks = pBlob->numPointmasses; 
  pBlob->numJoints = pBlob->numPointmasses * 5; 
  
  BS_MEM_ALLOC_TYPE(pBlob->pPointmasses, bs_pointmass_t*, pBlob->numPointmasses); 
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_float_t t; 
    
    t = (bs_float_t) i / (bs_float_t) pBlob->numPointmasses * M_PI * 2.0f; 
    pBlob->pPointmasses[i] = bs_pointmass_create(xPos + cos(t) * radius, yPos + sin(t) * radius, 1.0f); 
  }
  
  BS_MEM_ALLOC_TYPE(pBlob->pPointmassesScAccess, bs_pointmass_t*, pBlob->numPointmasses); 
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    pBlob->pPointmassesScAccess[i] = NULL; 
  }
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_int32_t j; 
    
    do
    {
      j = random() % pBlob->numPointmasses;    
    } while(pBlob->pPointmassesScAccess[j] != NULL);  
    
    pBlob->pPointmassesScAccess[j] = pBlob->pPointmasses[i]; 
  } 
  
  
  BS_MEM_ALLOC_TYPE(pBlob->pSticks, bs_stick_t*, pBlob->numSticks); 
  for(i = 1; i < pBlob->numSticks; i++)
  {
    pBlob->pSticks[i] = bs_stick_create(pBlob->pPointmasses[i - 1], pBlob->pPointmasses[i]); 
  }
  pBlob->pSticks[0] = bs_stick_create(pBlob->pPointmasses[0], pBlob->pPointmasses[pBlob->numSticks - 1]); 

  BS_MEM_ALLOC_TYPE(pBlob->pSticksScAccess, bs_stick_t*, pBlob->numSticks); 
  for(i = 0; i < pBlob->numSticks; i++)
  {
    pBlob->pSticksScAccess[i] = NULL; 
  }
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_int32_t j; 
    
    do 
    {
      j = random() % pBlob->numSticks;    
    } while(pBlob->pSticksScAccess[j] != NULL);      
    
    pBlob->pSticksScAccess[j] = pBlob->pSticks[i]; 
  } 
  
  
  pBlob->shortConst = 1.07; 
  pBlob->longConst = 1.45f; 
  BS_MEM_ALLOC_TYPE(pBlob->pJoints, bs_joint_t*, pBlob->numJoints); 
  for(i = 0, p = 0; i < pBlob->numPointmasses; i++)
  {
    bs_int32_t t, k; 

    k = pBlob->numPointmasses / 2; 

    t = bs_blob_clamp_index(i - 5, pBlob->numPointmasses); 
    pBlob->pJoints[p++] = bs_joint_create(pBlob->pPointmasses[i], 
      pBlob->pPointmasses[t], pBlob->shortConst, pBlob->longConst); 
    t = bs_blob_clamp_index(i + 5, pBlob->numPointmasses); 
    pBlob->pJoints[p++] = bs_joint_create(pBlob->pPointmasses[i],  
      pBlob->pPointmasses[t], pBlob->shortConst, pBlob->longConst); 

    t = bs_blob_clamp_index(i + k - 6, pBlob->numPointmasses); 
    pBlob->pJoints[p++] = bs_joint_create(pBlob->pPointmasses[i], 
      pBlob->pPointmasses[t], pBlob->shortConst, pBlob->longConst); 
    t = bs_blob_clamp_index(i + k + 6, pBlob->numPointmasses); 
    pBlob->pJoints[p++] = bs_joint_create(pBlob->pPointmasses[i], 
      pBlob->pPointmasses[t], pBlob->shortConst, pBlob->longConst); 
    
    pBlob->pJoints[p++] = bs_joint_create(pBlob->pPointmasses[i], 
      pBlob->pMiddle, pBlob->shortConst, pBlob->longConst); 
  }

  BS_MEM_ALLOC_TYPE(pBlob->pJointsScAccess, bs_joint_t*, pBlob->numJoints); 
  for(i = 0; i < pBlob->numJoints; i++)
  {
    pBlob->pJointsScAccess[i] = NULL; 
  }
  for(i = 0; i < pBlob->numJoints; i++)
  {
    bs_int32_t j; 
    
    do 
    {
      j = random() % pBlob->numJoints;    
    } while(pBlob->pJointsScAccess[j] != NULL); 
    
    pBlob->pJointsScAccess[j] = pBlob->pJoints[i]; 
  } 

  return pBlob; 
}

void 
bs_blob_destroy(bs_blob_t *pBlob)
{
  int i; 
  
  BS_ASSERT(pBlob != NULL); 

  bs_pointmass_destroy(pBlob->pMiddle); 
  
  BS_ASSERT(pBlob->pPointmasses != NULL);   
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_pointmass_destroy(pBlob->pPointmasses[i]); 
  }
  BS_MEM_FREE(pBlob->pPointmasses); 
  
  BS_ASSERT(pBlob->pSticks != NULL); 
  for(i = 0; i < pBlob->numSticks; i++)
  {
    bs_stick_destroy(pBlob->pSticks[i]); 
  }
  BS_MEM_FREE(pBlob->pSticks); 
  
  BS_ASSERT(pBlob->pJoints != NULL); 
  for(i = 0; i < pBlob->numJoints; i++)
  {
    bs_joint_destroy(pBlob->pJoints[i]); 
  }
  BS_MEM_FREE(pBlob->pJoints); 
  
  BS_MEM_FREE(pBlob); 
}

bs_float_t 
bs_blob_get_radius(bs_blob_t *pBlob)
{
  BS_ASSERT(pBlob != NULL); 
  return pBlob->radius;   
}

void
bs_blob_scale(bs_blob_t *pBlob, 
              bs_float_t scaleFactor)
{
  int i; 

  BS_ASSERT(pBlob != NULL); 
  BS_ASSERT(scaleFactor > 0.0f); 

  for(i = 0; i < pBlob->numJoints; i++)
  {
    bs_joint_scale(pBlob->pJoints[i], scaleFactor); 
  }    
  for(i = 0; i < pBlob->numSticks; i++)
  {
    bs_stick_scale(pBlob->pSticks[i], scaleFactor); 
  }
  pBlob->radius *= scaleFactor;
}

bs_pointmass_t* 
bs_blob_get_middle(bs_blob_t *pBlob)
{
  BS_ASSERT(pBlob != NULL);
  return pBlob->pMiddle;   
}

bs_float_t 
bs_blob_get_x_pos(bs_blob_t *pBlob)
{
  BS_ASSERT(pBlob != NULL);
  return bs_pointmass_get_x_pos(pBlob->pMiddle);   
}

bs_float_t 
bs_blob_get_y_pos(bs_blob_t *pBlob)
{
  BS_ASSERT(pBlob != NULL);
  return bs_pointmass_get_y_pos(pBlob->pMiddle);   
}

static bs_float_t 
bs_blob_compute_volume(bs_blob_t *pBlob)
{
  bs_int32_t i; 
  bs_float_t volume, t; 
  bs_vector_t *pA, *pB, *pM; 
  
  pM = bs_pointmass_get_pos(pBlob->pMiddle); 
  volume = 0.0f; 
  for(i = 0; i < pBlob->numSticks; i++)
  {
    pA = BS_STICK_GET_A_POS(pBlob->pSticks[i]); 
    pB = BS_STICK_GET_B_POS(pBlob->pSticks[i]); 
    
    t  = BS_VECTOR_GET_X(pA) * (BS_VECTOR_GET_Y(pB) - BS_VECTOR_GET_Y(pM)); 
    t += BS_VECTOR_GET_X(pB) * (BS_VECTOR_GET_Y(pM) - BS_VECTOR_GET_Y(pA)); 
    t += BS_VECTOR_GET_X(pM) * (BS_VECTOR_GET_Y(pA) - BS_VECTOR_GET_Y(pB)); 
    
    volume += fabs(t); 
  }
  
  volume *= 0.5f; 
  
  return volume; 
}

static void 
bs_blob_compute_middle(bs_blob_t *pBlob, 
                       bs_vector_t *pMiddle)
{
  bs_int32_t i; 

  BS_VECTOR_SET_X(pMiddle, 0.0f); 
  BS_VECTOR_SET_Y(pMiddle, 0.0f); 
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    BS_VECTOR_ADD(pMiddle, BS_POINTMASS_GET_CUR_POS(pBlob->pPointmasses[i])); 
  }    
  BS_VECTOR_SCALE(pMiddle, 1.0f / (bs_float_t) pBlob->numPointmasses);
}

static void 
bs_blob_pump(bs_blob_t *pBlob)
{
  bs_float_t targetVolume, actualVolume, scaleFactor; 
  bs_int32_t i; 
  bs_vector_t middle;
  
  targetVolume = pBlob->radius * pBlob->radius * M_PI * 1.5f; 
  actualVolume = bs_blob_compute_volume(pBlob);

  if(actualVolume < targetVolume * 0.0001f)
  {
    actualVolume = targetVolume * 0.0001f; 
  }
  
  scaleFactor = targetVolume / actualVolume; 
  scaleFactor -= 1.0f; 
  scaleFactor *= 0.1f; 
  
  bs_blob_compute_middle(pBlob, &middle); 
  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_vector_t t, *pPM; 
    
    pPM = bs_pointmass_get_pos(pBlob->pPointmasses[i]); 
    BS_VECTOR_SET(&t, pPM);
    BS_VECTOR_SUB(&t, &middle);
    BS_VECTOR_SCALE(&t, scaleFactor); 
    BS_VECTOR_ADD(pPM, &t);     
  }
}

void 
bs_blob_move(bs_blob_t *pBlob, 
             bs_float_t dt)
{
  int i; 

  BS_ASSERT(pBlob != NULL); 
  BS_ASSERT(pBlob->pPointmasses != NULL); 
  BS_ASSERT(pBlob->pMiddle != NULL); 
  
  bs_blob_compute_middle(pBlob, pBlob->pComputedMiddle);
  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_pointmass_move(pBlob->pPointmassesScAccess[i], dt); 
  }    
  bs_pointmass_move(pBlob->pMiddle, dt); 
}

void 
bs_blob_sc(bs_blob_t *pBlob, 
           bs_env_t *pEnv)
{
  int i, j;

  BS_ASSERT(pBlob != NULL); 
  BS_ASSERT(pBlob->pPointmasses != NULL); 
  BS_ASSERT(pBlob->pSticks != NULL); 
  BS_ASSERT(pBlob->pJoints != NULL); 

  for(j = 0; j < 10; j++)
  {
    for(i = 0; i < pBlob->numPointmasses; i++)
    {
      if(bs_env_collision(pEnv, bs_pointmass_get_pos(pBlob->pPointmassesScAccess[i]), 
         bs_pointmass_get_prev_pos(pBlob->pPointmassesScAccess[i])) == TRUE)
      {
        bs_pointmass_set_friction(pBlob->pPointmassesScAccess[i], 0.99f); 
      }
      else 
      {
        bs_pointmass_set_friction(pBlob->pPointmassesScAccess[i], 0.01f);         
      }
    }  
    for(i = 0; i < pBlob->numJoints; i++)
    {
      bs_joint_sc(pBlob->pJointsScAccess[i]); 
    }
    for(i = 0; i < pBlob->numSticks; i++)
    {
      bs_stick_sc(pBlob->pSticksScAccess[i]); 
    }  
    
    bs_blob_pump(pBlob); 
    
    for(i = pBlob->numPointmasses - 1; i >= 0; i--)
    {
      if(bs_env_collision(pEnv, bs_pointmass_get_pos(pBlob->pPointmassesScAccess[i]), 
         bs_pointmass_get_prev_pos(pBlob->pPointmassesScAccess[i])) == TRUE)
      {
        bs_pointmass_set_friction(pBlob->pPointmassesScAccess[i], 0.99f); 
      }
      else 
      {
        bs_pointmass_set_friction(pBlob->pPointmassesScAccess[i], 0.01f);         
      }
    }  
    for(i = pBlob->numJoints - 1; i >= 0; i--)
    {
      bs_joint_sc(pBlob->pJointsScAccess[i]); 
    }
    for(i = pBlob->numSticks - 1; i >= 0; i--)
    {
      bs_stick_sc(pBlob->pSticksScAccess[i]); 
    }  
    
    bs_blob_pump(pBlob); 
  }   
}

void 
bs_blob_set_force(bs_blob_t *pBlob, 
                  bs_vector_t *pForce)
{
  int i; 
  
  BS_ASSERT(pBlob != NULL); 
  BS_ASSERT(pBlob->pPointmasses != NULL); 
  BS_ASSERT(pBlob->pMiddle != NULL);  
  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_pointmass_set_force(pBlob->pPointmasses[i], pForce);  
  }    
  bs_pointmass_set_force(pBlob->pMiddle, pForce);  
}

void 
bs_blob_add_force(bs_blob_t *pBlob, 
                  bs_vector_t *pForce)
{
  int i; 

  BS_ASSERT(pBlob != NULL); 
  BS_ASSERT(pBlob->pPointmasses != NULL); 
  BS_ASSERT(pBlob->pMiddle != NULL);  
  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_pointmass_add_force(pBlob->pPointmasses[i], pForce);  
  }    
  bs_pointmass_add_force(pBlob->pMiddle, pForce);  
}

/* 
int pnpoly(int npol, float *xp, float *yp, float x, float y)
    {
      int i, j, c = 0;
      for (i = 0, j = npol-1; i < npol; j = i++) {
        if ((((yp[i]<=y) && (y<yp[j])) ||
             ((yp[j]<=y) && (y<yp[i]))) &&
            (x < (xp[j] - xp[i]) * (y - yp[i]) / (yp[j] - yp[i]) + xp[i]))

          c = !c;
      }
      return c;
    }
*/ 

/* 
            (Ay-Cy)(Dx-Cx)-(Ax-Cx)(Dy-Cy)
        r = -----------------------------  (eqn 1)
            (Bx-Ax)(Dy-Cy)-(By-Ay)(Dx-Cx)

            (Ay-Cy)(Bx-Ax)-(Ax-Cx)(By-Ay)
        s = -----------------------------  (eqn 2)
            (Bx-Ax)(Dy-Cy)-(By-Ay)(Dx-Cx)
            
        if 0<=r<=1 & 0<=s<=1, intersection exists
        
        then       P=A+r(B-A) or
 */ 

static bs_bool_t
bs_blob_intersect(bs_vector_t *pA, 
                  bs_vector_t *pB, 
                  bs_vector_t *pC, 
                  bs_vector_t *pD, 
                  bs_vector_t *pR)
{
  bs_float_t r, s, k; 
  bs_float_t aX, aY, bX, bY, cX, cY, dX, dY; 
  
  aX = BS_VECTOR_GET_X(pA); 
  aY = BS_VECTOR_GET_Y(pA); 
  bX = BS_VECTOR_GET_X(pB); 
  bY = BS_VECTOR_GET_Y(pB); 
  cX = BS_VECTOR_GET_X(pC); 
  cY = BS_VECTOR_GET_Y(pC); 
  dX = BS_VECTOR_GET_X(pD); 
  dY = BS_VECTOR_GET_Y(pD);
  
  k = ((bX-aX) * (dY-cY) - (bY-aY) * (dX-cX)); 
  if(k == 0.0f)
  {
    return FALSE; 
  }

  r = ((aY-cY) * (dX-cX) - (aX-cX) * (dY-cY)) / k; 
  s = ((aY-cY) * (bX-aX) - (aX-cX) * (bY-aY)) / k; 

    
  BS_VECTOR_SET(pR, pB); 
  BS_VECTOR_SUB(pR, pA);
  BS_VECTOR_SCALE(pR, r);
  BS_VECTOR_ADD(pR, pA); 
  
  return TRUE; 
}

static bs_bool_t 
bs_blob_inside_triangle(bs_vector_t *p0, // point 
                        bs_vector_t *p1, // triangle 
                        bs_vector_t *p2, // triangle 
                        bs_vector_t *p3) // triangle
{
  bs_float_t b0, b1, b2, b3; 
  bs_float_t x0, y0, x1, y1, x2, y2, x3, y3; 
  
  x0 = BS_VECTOR_GET_X(p0); 
  y0 = BS_VECTOR_GET_Y(p0); 
  x1 = BS_VECTOR_GET_X(p1); 
  y1 = BS_VECTOR_GET_Y(p1); 
  x2 = BS_VECTOR_GET_X(p2); 
  y2 = BS_VECTOR_GET_Y(p2); 
  x3 = BS_VECTOR_GET_X(p3); 
  y3 = BS_VECTOR_GET_Y(p3);
  
  b0 = ((x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1)); 
  b1 = (((x2 - x0) * (y3 - y0) - (x3 - x0) * (y2 - y0)) / b0); 
  b2 = (((x3 - x0) * (y1 - y0) - (x1 - x0) * (y3 - y0)) / b0); 
  b3 = 1.0f - b1 - b2; 
  
  if(b1 >= 0.0f && b2 >= 0.0f && b3 >= 0.0f)
  {
    return TRUE; 
  }
  
  return FALSE; 
}

static void 
bs_blob_test_point_blob(bs_blob_t *pBlob,   
                        bs_pointmass_t *pPm, 
                        bs_vector_t *pMiddle)
{
  int i, j, np; 
  bs_vector_t *pCur; 
    
  pCur = BS_POINTMASS_GET_CUR_POS(pPm);
  np = pBlob->numPointmasses;  
  for(i = 0, j = np - 1; i < np; j = i, i++)
  {
    bs_bool_t intersect;
    bs_vector_t *pA, *pB;     
    
    pA = BS_POINTMASS_GET_CUR_POS(pBlob->pPointmasses[j]); 
    pB = BS_POINTMASS_GET_CUR_POS(pBlob->pPointmasses[i]); 
    intersect = bs_blob_inside_triangle(pCur, pBlob->pComputedMiddle, pA, pB); 
    
    if(intersect == TRUE)
    {
      bs_vector_t d; 
      
      if(bs_blob_intersect(pCur, pBlob->pComputedMiddle, pA, pB, &d) == FALSE)
      {
        continue;   
      }
      
      BS_VECTOR_SUB(&d, pCur);
      BS_VECTOR_SCALE(&d, 0.05f); 
      BS_VECTOR_ADD(pCur, &d); 
      
      bs_pointmass_set_friction(pPm, 0.25f); 
      bs_pointmass_set_rgb(pPm, 0.0f, 0.0f, 1.0f); 
      
      return;
    }
  }
}

void 
bs_blob_collision_test_(bs_blob_t *pA, 
                           bs_blob_t *pB)
{
  int i; 
  bs_float_t aRadius, bRadius, dist; 
  bs_vector_t *pAMid, *pBMid; 
  
  /* 
  aRadius = bs_blob_get_radius(pA); 
  bRadius = bs_blob_get_radius(pB); 
  pAMid = BS_POINTMASS_GET_CUR_POS(pA->pMiddle); 
  pBMid = BS_POINTMASS_GET_CUR_POS(pB->pMiddle); 
  dist = BS_VECTOR_DIST(pAMid, pBMid); 
  if(dist > 1.5f * (aRadius + bRadius))
  {
    return; 
  }
  */
  
  for(i = 0; i < pA->numPointmasses; i++)
  {
    bs_blob_test_point_blob(pB, pA->pPointmasses[i], pA->pComputedMiddle); 
  }    
}

//////////////////////////

/* Compute the separation plane between two blobs, result is defined by 
 * pResultA and pResultB. This function is the reason why blobs interesct 
 * in 'strange' fashion. The separation plane tries to find the line at which 
 * two blobs start overlapping ewach other. This would work if the blobs where 
 * perfect circles, but they are bobs, thus deforming and the separation plane 
 * becomes a mere approximation. 
 */ 
static void 
bs_blob_find_separation_plane(bs_blob_t *pBlobA, 
                              bs_blob_t *pBlobB, 
                              bs_vector_t *pResultA, 
                              bs_vector_t *pResultB)
{
  bs_float_t tx, ty, px, py; 
  bs_float_t ax, ay; 
  bs_float_t k, kA, kB; 

  /* M_PI constant removed, kA and kB are only valid for k computation */ 
  /* FIXME, get rid of sqrt by using one step taylor or something like that */ 
  kA = sqrt(bs_blob_compute_volume(pBlobA)); 
  kB = sqrt(bs_blob_compute_volume(pBlobB)); 
  k = kA / (kA + kB); 
  
  ax = BS_VECTOR_GET_X(pBlobA->pComputedMiddle); 
  ay = BS_VECTOR_GET_Y(pBlobA->pComputedMiddle); 
  
  tx = BS_VECTOR_GET_X(pBlobB->pComputedMiddle) - ax; 
  ty = BS_VECTOR_GET_Y(pBlobB->pComputedMiddle) - ay; 
    
  px = - ty;   
  py = tx; 
  
  /* reuse tx, ty */
  tx = tx * k + ax;   
  ty = ty * k + ay;   

  /* reuse px, py */
  px = px * 0.5f; 
  py = py * 0.5f; 
  
  BS_VECTOR_SET_X(pResultA, tx + px);
  BS_VECTOR_SET_Y(pResultA, ty + py);

  BS_VECTOR_SET_X(pResultB, tx - px); 
  BS_VECTOR_SET_Y(pResultB, ty - py); 
}

/* Test is the line segment starting in pSegmentA ending in pSegmentB 
 * intersects with an infinite line defined by pLineA and pLineB. 
 * returns FALSE if they do not intersect, in that case 
 * the value of pResult is undefined. 
 */ 
static bs_bool_t
bs_blob_plane_intersect(bs_vector_t *pLineA, 
                        bs_vector_t *pLineB, 
                        bs_vector_t *pSegmentA, 
                        bs_vector_t *pSegmentB, 
                        bs_vector_t *pResult)
{
  bs_float_t s, k; 
  bs_float_t aX, aY, bX, bY, cX, cY, dX, dY; 
  
  aX = BS_VECTOR_GET_X(pLineA); 
  aY = BS_VECTOR_GET_Y(pLineA); 
  bX = BS_VECTOR_GET_X(pLineB); 
  bY = BS_VECTOR_GET_Y(pLineB); 
  cX = BS_VECTOR_GET_X(pSegmentA); 
  cY = BS_VECTOR_GET_Y(pSegmentA); 
  dX = BS_VECTOR_GET_X(pSegmentB); 
  dY = BS_VECTOR_GET_Y(pSegmentB);
  
  /* the lines are paralle if k is zero */ 
  k = ((bX-aX) * (dY-cY) - (bY-aY) * (dX-cX)); 
  if(k == 0.0f)
  {
    return FALSE; 
  }

  s = ((aY-cY) * (bX-aX) - (aX-cX) * (bY-aY)) / k; 

  if(s < 0.0f || s > 1.0f) 
  {
    return FALSE; 
  }

  BS_VECTOR_SET(pResult, pSegmentB); 
  BS_VECTOR_SUB(pResult, pSegmentA);
  BS_VECTOR_SCALE(pResult, s);
  BS_VECTOR_ADD(pResult, pSegmentA); 
  
  return TRUE; 
}

/* Test is two blobs can possibly intersect */ 
static bs_bool_t 
bs_blob_simple_distance_test(bs_blob_t *pBlobA, 
                             bs_blob_t *pBlobB)
{
  bs_float_t aRadius, bRadius, dist; 
  
  aRadius = bs_blob_get_radius(pBlobA); 
  bRadius = bs_blob_get_radius(pBlobB); 
  dist = BS_VECTOR_DIST(pBlobA->pComputedMiddle, pBlobB->pComputedMiddle); 
  if(dist > 1.5f * (aRadius + bRadius))
  {
    return FALSE;; 
  }
  
  return TRUE; 
}

static bs_bool_t 
bs_blob_point_inside_blob(bs_blob_t *pBlob, 
                          bs_vector_t *pPoint)
{
  bs_int32_t i, j; 
  bs_vector_t *pPMA, *pPMB, *pMiddle; 
  
  pMiddle = pBlob->pComputedMiddle; 
  for(i = 0, j = pBlob->numPointmasses - 1; i < pBlob->numPointmasses; j = i, i++)
  {
    pPMA = BS_POINTMASS_GET_CUR_POS(pBlob->pPointmasses[i]); 
    pPMB = BS_POINTMASS_GET_CUR_POS(pBlob->pPointmasses[j]); 
    if(bs_blob_inside_triangle(pPoint, pMiddle, pPMA, pPMB) == TRUE)
    {
      return TRUE;   
    }
  }
  return FALSE; 
}

void 
bs_blob_collision_test(bs_blob_t *pBlobA, 
                       bs_blob_t *pBlobB)
{
  bs_int32_t i; 
  
  if(bs_blob_simple_distance_test(pBlobA, pBlobB) == FALSE)
  {
    return; 
  }
  
  bs_blob_find_separation_plane(pBlobA, pBlobB, pBlobA->spA, pBlobA->spB);   
  
  for(i = 0; i < pBlobA->numPointmasses; i++)
  {
    bs_vector_t r; 
    bs_vector_t *pTmp; 
    
    pTmp = BS_POINTMASS_GET_CUR_POS(pBlobA->pPointmasses[i]); 
    if(bs_blob_plane_intersect(pBlobA->spA, pBlobA->spB, pBlobA->pComputedMiddle, pTmp, &r) == TRUE)
    {
      //if(bs_blob_point_inside_blob(pBlobB, &r) == TRUE)
      //{
        BS_VECTOR_SET(pTmp, &r); 
        bs_pointmass_set_friction(pBlobA->pPointmasses[i], 0.1f);        
      //}
    }      
  }
}


static void 
bs_blob_randomize_face_style(bs_blob_t *pBlob)
{
  BS_ASSERT(pBlob != NULL);

  if(pBlob->faceStyle == BS_BLOB_FACE_STYLE_MOUTH_CLOSED && bs_util_random_float() < 0.05f)
  {
    pBlob->faceStyle = BS_BLOB_FACE_STYLE_MOUTH_OPEN; 
  }
  else if(pBlob->faceStyle == BS_BLOB_FACE_STYLE_MOUTH_OPEN && bs_util_random_float() < 0.1f)
  {
    pBlob->faceStyle = BS_BLOB_FACE_STYLE_MOUTH_CLOSED; 
  }
  
  if(pBlob->eyeStyle == BS_BLOB_EYE_STYLE_OPEN && bs_util_random_float() < 0.025f)
  {
    pBlob->eyeStyle = BS_BLOB_EYE_STYLE_BLINK; 
  }
  else if(pBlob->eyeStyle == BS_BLOB_EYE_STYLE_BLINK && bs_util_random_float() < 0.3f)
  {
    pBlob->eyeStyle = BS_BLOB_EYE_STYLE_OPEN; 
  }
}

static void 
bs_blob_draw_mouth_closed(bs_blob_t *pBlob, 
                          cairo_t *pCairo, 
                          bs_float_t rsf)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  cairo_new_path(pCairo); 
  cairo_set_line_width(pCairo, 2.0 / rsf);   
  cairo_arc(pCairo, 0.0, 0.0, 0.25f, M_PI, FALSE); 
  cairo_stroke(pCairo); 
}

static void 
bs_blob_draw_mouth_open(bs_blob_t *pBlob, 
                        cairo_t *pCairo, 
                        bs_float_t rsf)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  cairo_new_path(pCairo); 
  cairo_set_line_width(pCairo, 2.0 / rsf);   
  cairo_arc(pCairo, 0.0, 0.0, 0.25f, M_PI, FALSE); 
  cairo_close_path(pCairo); 
  cairo_fill(pCairo); 
}

static void 
bs_blob_draw_eyes_open(bs_blob_t *pBlob, 
                       cairo_t *pCairo, 
                       bs_float_t rsf)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  cairo_set_line_width(pCairo, 1.0 / rsf);
  
  cairo_new_path(pCairo); 
  cairo_arc(pCairo, -0.15, 0.20, 0.12, 0.0, 2.0 * M_PI); 
  cairo_close_path(pCairo); 
  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0);   
  cairo_fill_preserve(pCairo); 
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0);   
  cairo_stroke(pCairo); 
    
  cairo_new_path(pCairo); 
  cairo_arc(pCairo, 0.15, 0.20, 0.12, 0.0, 2.0 * M_PI); 
  cairo_close_path(pCairo); 
  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0);   
  cairo_fill_preserve(pCairo); 
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0);   
  cairo_stroke(pCairo); 
  
  cairo_new_path(pCairo);
  cairo_arc(pCairo, -0.15, 0.17, 0.06, 0.0, 2.0 * M_PI); 
  cairo_close_path(pCairo);
  cairo_fill(pCairo);   

  cairo_new_path(pCairo);
  cairo_arc(pCairo, 0.15, 0.17, 0.06, 0.0, 2.0 * M_PI); 
  cairo_close_path(pCairo);
  cairo_fill(pCairo);   
}

static void 
bs_blob_draw_eyes_blink(bs_blob_t *pBlob, 
                        cairo_t *pCairo, 
                        bs_float_t rsf)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  cairo_set_line_width(pCairo, 1.0 / rsf);

  cairo_new_path(pCairo); 
  cairo_arc(pCairo, -0.15, 0.20, 0.12, 0.0, 2.0 * M_PI); 
  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0);   
  cairo_fill_preserve(pCairo); 
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0);   
  cairo_stroke(pCairo); 
    
  cairo_new_path(pCairo); 
  cairo_arc(pCairo, 0.15, 0.20, 0.12, 0.0, 2.0 * M_PI); 
  cairo_close_path(pCairo);
  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0);   
  cairo_fill_preserve(pCairo); 
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0);   
  cairo_stroke(pCairo); 

  cairo_new_path(pCairo); 
  cairo_move_to(pCairo, -0.25, 0.20);
  cairo_line_to(pCairo, -0.05, 0.20); 
  cairo_stroke(pCairo);  

  cairo_new_path(pCairo); 
  cairo_move_to(pCairo,  0.25, 0.20);
  cairo_line_to(pCairo,  0.05, 0.20); 
  cairo_stroke(pCairo);   
}

static void 
bs_blob_draw_ooh_face(bs_blob_t *pBlob, 
                      cairo_t *pCairo, 
                      bs_float_t rsf)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);
  
  bs_blob_draw_mouth_open(pBlob, pCairo, rsf); 
  
  cairo_set_line_width(pCairo, 2.0 / rsf); 
  
  cairo_new_path(pCairo); 
  cairo_move_to(pCairo, -0.25, 0.3); 
  cairo_line_to(pCairo, -0.05, 0.2); 
  cairo_line_to(pCairo, -0.25, 0.1); 
  cairo_stroke(pCairo); 

  cairo_new_path(pCairo); 
  cairo_move_to(pCairo,  0.25, 0.3); 
  cairo_line_to(pCairo,  0.05, 0.2); 
  cairo_line_to(pCairo,  0.25, 0.1); 
  cairo_stroke(pCairo); 
}

static void 
bs_blob_draw_face(bs_blob_t *pBlob, 
                  cairo_t *pCairo, 
                  bs_float_t scaleFactor)
{
  bs_float_t xOff, yOff; 
  bs_vector_t up, orientation; 
  bs_float_t angle; 
  bs_float_t rsf; 

  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  rsf = pBlob->radius * scaleFactor; 
  
  cairo_save(pCairo); 
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0); 
  
  BS_VECTOR_SET(&orientation, bs_pointmass_get_pos(pBlob->pPointmasses[0])); 
  BS_VECTOR_ADD(&orientation, bs_pointmass_get_pos(pBlob->pPointmasses[1])); 
  BS_VECTOR_SUB(&orientation, bs_pointmass_get_pos(pBlob->pPointmasses[2])); 
  BS_VECTOR_SUB(&orientation, bs_pointmass_get_pos(pBlob->pPointmasses[3])); 
  
  xOff = (bs_pointmass_get_x_pos(pBlob->pMiddle) + BS_VECTOR_GET_X(&orientation)) * scaleFactor; 
  yOff = (bs_pointmass_get_y_pos(pBlob->pMiddle) + BS_VECTOR_GET_Y(&orientation)) * scaleFactor;
  cairo_translate(pCairo, xOff, yOff); 
  
  bs_vector_init(&up, 0.0f, 1.0f); 
  angle = acos(bs_vector_dot_prod(&orientation, &up) / bs_vector_length(&orientation)); 
  if(bs_vector_get_x(&orientation) > 0.0f)
  {
    angle = -angle; 
  }
  cairo_rotate(pCairo, angle); 
 
  /* face offset relaitve middle position */
  /* face off is a crap movie btw */   
  cairo_scale(pCairo, rsf, rsf); 
  cairo_translate(pCairo, 0.0, -0.30); 

  bs_blob_randomize_face_style(pBlob); 
  
  if(bs_pointmass_get_square_vel(pBlob->pMiddle) > 0.004f)
  {
    bs_blob_draw_ooh_face(pBlob, pCairo, rsf); 
  }
  else 
  {
    if(pBlob->faceStyle == BS_BLOB_FACE_STYLE_MOUTH_CLOSED)
    {
      bs_blob_draw_mouth_closed(pBlob, pCairo, rsf); 
    }
    else if(pBlob->faceStyle == BS_BLOB_FACE_STYLE_MOUTH_OPEN)
    {
      bs_blob_draw_mouth_open(pBlob, pCairo, rsf);       
    }
    else 
    {
      BS_ASSERT(FALSE);   
    }
    
    if(pBlob->eyeStyle == BS_BLOB_EYE_STYLE_BLINK)
    {    
      bs_blob_draw_eyes_blink(pBlob, pCairo, rsf); 
    }
    else if(pBlob->eyeStyle == BS_BLOB_EYE_STYLE_OPEN)
    {
      bs_blob_draw_eyes_open(pBlob, pCairo, rsf);       
    }
    else 
    {
      BS_ASSERT(FALSE);   
    }
  }
  
  cairo_restore(pCairo); 
}

static void 
bs_blob_draw_point(bs_vector_t *pPoint, 
                   cairo_t *pCairo, 
                   bs_float_t scaleFactor)
{
  cairo_set_source_rgb(pCairo, 0.0f, 1.0f, 0.0f); 
  cairo_set_line_width(pCairo, 1.0f); 
  
  cairo_new_path(pCairo); 
  cairo_arc(pCairo, BS_VECTOR_GET_X(pPoint) * scaleFactor, 
    BS_VECTOR_GET_Y(pPoint) * scaleFactor, 5.0f, 0.0f, M_PI * 2.0f); 
  cairo_fill(pCairo); 
}

static void 
bs_blob_draw_simple_body(bs_blob_t *pBlob, 
                         cairo_t *pCairo, 
                         bs_float_t scaleFactor)
{
  bs_int32_t i; 
  
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);
  
  cairo_new_path(pCairo); 

  cairo_move_to(pCairo, 
    (bs_pointmass_get_x_pos(pBlob->pPointmasses[0]) * scaleFactor), 
    (bs_pointmass_get_y_pos(pBlob->pPointmasses[0]) * scaleFactor));      

  for(i = 1; i < pBlob->numPointmasses; i++)
  {
    cairo_line_to(pCairo, 
      (bs_pointmass_get_x_pos(pBlob->pPointmasses[i]) * scaleFactor), 
      (bs_pointmass_get_y_pos(pBlob->pPointmasses[i]) * scaleFactor));      
  }
    
  cairo_close_path(pCairo); 

#ifdef BS_BLOB_DEBUG_FILL_BLOB
  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0); 
  cairo_fill_preserve(pCairo); 
#endif /* BS_BLOB_DEBUG_FILL_BLOB */ 
  
  cairo_set_line_width(pCairo, 3.0);   
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0); 
  cairo_stroke(pCairo);   

#ifdef BS_BLOB_DEBUG_DRAW_JOINTS
  for(i = 0; i < pBlob->numJoints; i++)
  {
    bs_joint_draw(pBlob->pJoints[i], pCairo, scaleFactor); 
  } 
#endif /* BS_BLOB_DEBUG_DRAW_JOINTS */ 
  
#ifdef BS_BLOB_DEBUG_DRAW_POINTMASSES  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    bs_pointmass_draw(pBlob->pPointmasses[i], pCairo, scaleFactor); 
    bs_pointmass_set_rgb(pBlob->pPointmasses[i], 0.2f, 1.0f, 0.0f);
  }
#endif /* BS_BLOB_DEBUG_DRAW_POINTMASSES */  

#ifdef BS_BLOB_DEBUG_DRAW_SEPATATION_PLANE
  cairo_new_path(pCairo); 
  cairo_move_to(pCairo, BS_VECTOR_GET_X(pBlob->spA) * scaleFactor, 
    BS_VECTOR_GET_Y(pBlob->spA) * scaleFactor);  
  cairo_line_to(pCairo, BS_VECTOR_GET_X(pBlob->spB) * scaleFactor, 
    BS_VECTOR_GET_Y(pBlob->spB) * scaleFactor);  
  cairo_close_path(pCairo); 
    
  cairo_set_line_width(pCairo, 1.0);   
  cairo_set_source_rgb(pCairo, 0.0, 0.8, 0.8); 
  cairo_stroke(pCairo);   
#endif /* BS_BLOB_DEBUG_DRAW_SEPATATION_PLANE */ 
}

/* no longer used ... but I suppose I can keep it for a while 

#define CLAMP_INDEX(I) ((I) & 0x03)

static void 
bs_blob_draw_body(bs_blob_t *pBlob, 
                  cairo_t *pCairo, 
                  bs_float_t scaleFactor)
{
  bs_int32_t i; 
  bs_float_t px, py, nx, ny, tx, ty, cx, cy; 
  bs_pointmass_t **pPointmasses; 
  
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  pPointmasses = pBlob->pPointmasses; 
  
  cairo_new_path(pCairo); 

  cairo_move_to(pCairo, 
    BS_POINTMASS_GET_X_POS(pPointmasses[0]) * scaleFactor, 
    BS_POINTMASS_GET_Y_POS(pPointmasses[0]) * scaleFactor);  
  
  for(i = 0; i < pBlob->numPointmasses; i++)
  {
    tx = BS_POINTMASS_GET_X_POS(pPointmasses[CLAMP_INDEX(i + 1)]); 
    ty = BS_POINTMASS_GET_Y_POS(pPointmasses[CLAMP_INDEX(i + 1)]); 
    
    cx = BS_POINTMASS_GET_X_POS(pPointmasses[i]); 
    cy = BS_POINTMASS_GET_Y_POS(pPointmasses[i]); 
    
    px = (cx + tx) * 0.5f; 
    py = (cy + ty) * 0.5f;
    
    nx = cx - BS_POINTMASS_GET_X_POS(pPointmasses[CLAMP_INDEX(i + 3)]);
    nx = nx + tx - BS_POINTMASS_GET_X_POS(pPointmasses[CLAMP_INDEX(i + 2)]); 
    ny = cy - BS_POINTMASS_GET_Y_POS(pPointmasses[CLAMP_INDEX(i + 3)]); 
    ny = ny + ty - BS_POINTMASS_GET_Y_POS(pPointmasses[CLAMP_INDEX(i + 2)]); 
    
    px += nx * 0.25f; 
    py += ny * 0.25f;
    
    px *= scaleFactor; 
    py *= scaleFactor; 
    
    tx *= scaleFactor; 
    ty *= scaleFactor; 
    
    cairo_curve_to(pCairo, px, py, tx, ty, tx, ty);     
  }

  cairo_close_path(pCairo); 

  cairo_set_source_rgb(pCairo, 1.0, 1.0, 1.0); 
  cairo_fill_preserve(pCairo); 

  cairo_set_line_width(pCairo, 3.0);   
  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0); 
  cairo_stroke(pCairo); 
}

*/

void 
bs_blob_draw(bs_blob_t *pBlob, 
             cairo_t *pCairo, 
             bs_float_t scaleFactor)
{
  BS_ASSERT(pBlob != NULL);
  BS_ASSERT(pCairo != NULL);

  cairo_set_source_rgb(pCairo, 0.0, 0.0, 0.0); 
  
  bs_blob_draw_simple_body(pBlob, pCairo, scaleFactor); 
  
  bs_blob_draw_face(pBlob, pCairo, scaleFactor); 

  if (cairo_status(pCairo)) 
  {
    printf("Cairo is unhappy: %s\n",
      cairo_status_to_string (cairo_status (pCairo)));
    exit(0);
  }
}
